#include <virtuabotixRTC.h>
// Horloge temps réel
// SCLK -> 6, I/O -> 7, CE -> 8
// CLK -> 6 , DAT -> 7, Reset -> 8
virtuabotixRTC myRTC(6, 7, 8);

#include <Adafruit_NeoPixel.h>
// Bandeau 121 leds
#define PIN 9          // On connecte le bandeau de LED sur la broche 9
#define NUMPIXELS 121  // On a 121 LEDS sur le bandeau
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

// Bouton de de réglage de l'horloge
// bouton 1 -> Mode setup heure (en rouge) -> mode setup minutes (en vert) -> Mise à jour RTC et fin
// bouton 2 -> +1 heure ou + 1mn
const int bouton1Pin = 2;
const int bouton2Pin = 4;
int bouton1Etat = 0;
int bouton2Etat = 0;

// Capteur de lumière
int capteur_lum = 0;       // capteur branché sur le port A0
int analog_lum;            // valeur analogique envoyé par le capteur
int intensite = 255;       // Intensité des leds
const int iMax = 255;      // Intensité max si beaucoup de lumière
const int iMin = 20;       // Intensité min si nuit noire
const int lMin = 20;       // Luminosité lue en pleine nuit
const int lMax = 800;      // Luminosité lue en pleine lumière

// Définition des leds à allumer pour chaque valeur trouvée
const int htexte[13][13] = {
  { 39, 38, 37, 36, 35, 34, NULL },                          // MINUIT
  { 50, 51, 52, 61, 60, 59, 58, 57, NULL },                  // UNE HEURE
  { 8, 9, 10, 11, 61, 60, 59, 58, 57, 56, NULL },            // DEUX HEURES
  { 19, 18, 17, 16, 15, 61, 60, 59, 58, 57, 56, NULL },      // TROIS HEURES
  { 23, 24, 25, 26, 27, 28, 61, 60, 59, 58, 57, 56, NULL },  // QUATRE HEURES
  { 45, 46, 47, 48, 61, 60, 59, 58, 57, 56, NULL },          // CINQ HEURES
  { 22, 21, 20, 61, 60, 59, 58, 57, 56, NULL },              // SIX HEURES
  { 15, 14, 13, 12, 61, 60, 59, 58, 57, 56, NULL },          // SEPT HEURES
  { 66, 65, 64, 63, 61, 60, 59, 58, 57, 56, NULL },          // HUIT HEURES
  { 51, 52, 53, 54, 61, 60, 59, 58, 57, 56, NULL },          // NEUF HEURES
  { 42, 41, 40, 61, 60, 59, 58, 57, 56, NULL },              // DIX HEURES
  { 30, 31, 32, 33, 61, 60, 59, 58, 57, 56, NULL },          // ONZE HEURES
  { 44, 43, 42, 41, NULL }                                   // MIDI
};

const int ilEst[6] = { 1, 2, 4, 5, 6, NULL };                                              // IL EST
const int duMatin[8] = { 106, 105, 111, 112, 113, 114, 115, NULL };                        // DU MATIN
const int duSoir[7] = { 106, 105, 103, 102, 101, 100, NULL };                              // DU SOIR
const int m5[5] = { 95, 96, 97, 98, NULL };                                                // CINQ
const int m10[4] = { 110, 109, 108, NULL };                                                // DIX
const int m15[8] = { 72, 73, 82, 81, 80, 79, 78, NULL };                                   // ET QUART
const int m20[6] = { 89, 90, 91, 92, 93, NULL };                                           // VINGT
const int m25[11] = { 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, NULL };                      // VINGT-CINQ
const int m30[8] = { 72, 73, 88, 87, 86, 85, 84, NULL };                                   // ET DEMIE
const int m35[16] = { 67, 68, 69, 70, 71, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, NULL };  // MOINS VINGT-CINQ
const int m40[11] = { 67, 68, 69, 70, 71, 89, 90, 91, 92, 93, NULL };                      // MOINS VINGT
const int m45[13] = { 67, 68, 69, 70, 71, 76, 77, 82, 81, 80, 79, 78, NULL };              // MOINS LE QUART
const int m50[9] = { 67, 68, 69, 70, 71, 110, 109, 108, NULL };                            // MOINS DIX
const int m55[10] = { 67, 68, 69, 70, 71, 95, 96, 97, 98, NULL };                          // MOINS CINQ
const int p1[3] = { 117, 118, NULL };                                                      // +1
const int p2[3] = { 117, 119, NULL };                                                      // +2
const int p3[3] = { 117, 120, NULL };                                                      // +3
const int p4[3] = { 117, 121, NULL };                                                      // +4


byte chaine[NUMPIXELS] = { 0 };
int modeSetup = 0;

// Initialisation des variables du temps
int heure = 0;
int minute = 0;

void setup() {
  Serial.begin(9600);
  pixels.begin();
  pixels.clear();  // Remet tous les pixel à zéro
  pinMode(bouton1Pin, INPUT);
  pinMode(bouton2Pin, INPUT);
}

// Mise à jour de l'état du tableau chaine qui pilotera les leds
void allumeLed(int px[], String texte) {
  Serial.print(texte + " ");
  for (int i = 0; px[i] != NULL; i++) {
    chaine[px[i] - 1] = true;
  }
}

// Délai avec test des boutons
void _delay(unsigned long d) {
  bouton1Etat = 0;
  bouton2Etat = 0;
  unsigned long start = millis();
  while (millis() - start < d) {
    if (digitalRead(bouton1Pin) == HIGH) {
      Serial.println("Bouton 1 Pressé");
      bouton1Etat = 1;
      break;
    }
    if (digitalRead(bouton2Pin) == HIGH) {
      Serial.println("Bouton 2 Pressé");
      bouton2Etat = 1;
      break;
    }
  }
}

void loop() {

  // On lit l'heure
  myRTC.updateTime();
  int heureRTC = myRTC.hours;
  int minuteRTC = myRTC.minutes;

  // On lit la luminosité
  analog_lum = analogRead(capteur_lum); 
  // On calcule l'intensité des leds en fonction de la luminosité extérieure  
  long c1 = analog_lum  - lMin;
  long c2 = iMax - iMin;
  long c3 = lMax - lMin;
  intensite = round(c1 * c2 / c3);  
  if(intensite < iMin ) { intensite = iMin; }
  if(intensite > iMax ) { intensite = iMax; }


  Serial.print("Valeur luminosité = ");  
  Serial.print(analog_lum); 
  Serial.print(" - Intensité = ");
  Serial.println(intensite); 
  
  // Pas de transition si on est en mode setup
  if (modeSetup == 0) {
    heure = heureRTC;
    minute = minuteRTC;
    // On fait disparaitre le texte en fade
    for (int lum = intensite; lum >= 0; lum = lum - 1) {
      for (int i = NUMPIXELS; i >= 0; i--) {
        if (chaine[i]) {
          pixels.setPixelColor((i), pixels.Color(lum, lum, lum));      
        }
      }
      pixels.show();
      // Le délai est calculé pour compenser le temps de la boucle (entre 0 et 12)
      delay((iMax-intensite)/iMin);
    }
    _delay(1000);
    // Toutes les heures, on met une animation
    if(minuteRTC==0) {
      for(int j = 0; j < 90; j++) {
        for (int i = 0; i < NUMPIXELS; i++) {
          pixels.setPixelColor((i), pixels.Color(random(256), random(256), random(256)));          
        }    
        pixels.show();
      }
    }
  }
  // On remet le tableau de pixels à 0
  for (int i = 0; i < NUMPIXELS; i++) {
    chaine[i] = false;
  }
  // On remet toutes les LED à l'état éteint
  pixels.clear();

  Serial.print(heure);
  Serial.print(":");
  Serial.println(minute);

  // Début de l'affichage
  allumeLed(ilEst, "IL EST");

  int heureaff = heure;
  // Passé 30 minutes, On ajoute une heure et les minutes passent en "moins xxx".
  if (minute >= 35) {
    heureaff++;
  }
  // Passé midi, on reprend le texte des heures du matin
  if (heureaff > 12) {
    heureaff = heureaff - 12;
  }
  // Traitement du cas de minuit
  if (heure == 23 && heureaff == 12) { heureaff = 0; }

  Serial.print(heureaff);
  Serial.print(":");
  Serial.println(minute);

  allumeLed(htexte[heureaff], String("") + heureaff + " HEURES");

  // On traite les minutes de 5 en 5
  if (minute >= 5 and minute < 10) { allumeLed(m5, "CINQ"); }
  if (minute >= 10 and minute < 15) { allumeLed(m10, "DIX"); }
  if (minute >= 15 and minute < 20) { allumeLed(m15, "ET QUART"); }
  if (minute >= 20 and minute < 25) { allumeLed(m20, "VINGT"); }
  if (minute >= 25 and minute < 30) { allumeLed(m25, "VINGT-CINQ"); }
  if (minute >= 30 and minute < 35) { allumeLed(m30, "ET DEMIE"); }
  if (minute >= 35 and minute < 40) { allumeLed(m35, "MOINS VINGT-CINQ"); }
  if (minute >= 40 and minute < 45) { allumeLed(m40, "MOINS VINGT"); }
  if (minute >= 45 and minute < 50) { allumeLed(m45, "MOINS LE QUART"); }
  if (minute >= 50 and minute < 55) { allumeLed(m50, "MOINS DIX"); }
  if (minute >= 55) { allumeLed(m55, "MOINS CINQ"); }

  if (heure > 0 and heure < 11) { allumeLed(duMatin, "DU MATIN"); }
  if (heure >= 20 and heure <= 23 and heureaff != 0) { allumeLed(duSoir, "DU SOIR"); }

  int plus = minute % 5;
  if (plus == 1) { allumeLed(p1, "PLUS UN"); }
  if (plus == 2) { allumeLed(p2, "PLUS DEUX"); }
  if (plus == 3) { allumeLed(p3, "PLUS TROIS"); }
  if (plus == 4) { allumeLed(p4, "PLUS QUATRE"); }
  Serial.println("");

  if (modeSetup == 0) {
    // On éclaire tous les bits à 1
    for (int i = 0; i < NUMPIXELS; i++) {
      // Une ligne sur deux (ligne impaire), on traite l'ordre à l'envers
      int j = i;
      if ((i - (i % 11)) % 2 != 0) {
        j = (2 * (i - (i % 11))) + 10 - i;
      }      
      if (chaine[j]) {
        pixels.setPixelColor((j), pixels.Color(intensite, intensite, intensite));
        pixels.show();
        delay(15);
      }
    }
  } else {
    for (int i = 0; i < NUMPIXELS; i++) {
      if (chaine[i]) {
        if (modeSetup == 1) {
          pixels.setPixelColor(i, pixels.Color(255, 0, 0));
        } else {
          pixels.setPixelColor(i, pixels.Color(0, 255, 0));
        }
      }
    }
    pixels.show();
  }
  Serial.println("");

  _delay(60000);

  // Si on a appuyé sur le bouton SETUP on bascule en mode setup heure puis setup minutes
  if (bouton1Etat == 1) {
    if (modeSetup == 0) {
      Serial.println("On passe en mode Setup Heure");
      modeSetup = 1;
    } else {
      if (modeSetup == 1) {
        Serial.println("On passe en mode Setup Minute");
        modeSetup = 2;
      } else {
        Serial.println("On sort du mode Setup et on met à jour l'horloge RTC");
        // Format date:
        // secondes, minutes, heures, jour de la semaine, jour du mois, mois, année
        myRTC.setDS1302Time(00, minute, heure, 1, 01, 01, 2000);
        modeSetup = 0;
      }
    }
    delay(500);
  }
  // Si on est en mode setup et qu'on appuie sur le bouton SET, on augmente le temps de 1MN
  if (bouton2Etat == 1) {
    if (modeSetup == 1) {
      heure++;
      delay(300);
    }
    if (modeSetup == 2) {
      minute++;
      delay(50);
    }
    if (minute > 59) {
      minute = 0;
      heure++;
    }
    if (heure > 23) {
      heure = 0;
    }
  }
}